-- Vicuna Exporter is a 3dsmax 6.0 utility to export data to the Vicuna Engine, by Carlos Valcrcel
-- The scene materials must be scripted plugins Vicuna Materials

utility ObjectExporter "Object_Exporter"
(

function MapName map=
(
	if classof map == BitmapTexture 
	then (filenameFromPath map.fileName) 
	else ("none.png")
)
function HasMap map=
(
	if classof map == BitmapTexture 
	then (1) 
	else (0)
)


function ExportMaterials Matlib Fichier =
(
	format "nmaterial %\n" Matlib.count to: Fichier
	format "\n" to: Fichier

	for mat in MatLib do
	(
	
		format "name %\n" (mat.name) to: Fichier
		format "col % % % %\n" (mat.diffuse.red/255.) (mat.diffuse.green/255.) (mat.diffuse.blue/255.) (.5)to: Fichier

		format "map %\n" (HasMap mat.diffuseMap) to: Fichier
		if( (HasMap mat.diffuseMap) ==1)
		then format "%\n" (MapName mat.diffuseMap) to: Fichier

		format "\n" to: Fichier
	)
)

function GetMatId Mat MatList =
(
	r=-0
	for i=1 to MatList.count do
	(
		Curr= MatList[i];
		
		try
		(
			if Curr.name == Mat.name
			then (r=i)
		)
		catch
		(
		)
	)
	(r)
)

function Export Obj Materials Fichier =
(
	tmesh = snapshotAsMesh obj

	numfaces = tmesh.numfaces
	numverts = getNumVerts tmesh
	numtverts = getNumTVerts tmesh
	
	matID = GetMatID Obj.material Materials
	
	format "mat_id %\n" (matID-1) to: Fichier
	
	format "name %\n" (obj.name) to: Fichier
	
	format "nvertex %\n" numverts to: Fichier
	
	pivotpos = obj.pivot
	format "position % % %\n" pivotpos.x pivotpos.y pivotpos.z to: Fichier
	
	for i=1 to numverts do
	(
		vertex = getVert tmesh i
		vx = (in coordsys world vertex.x)
		vy = (in coordsys world vertex.y)
		vz = (in coordsys world vertex.z)
		
		format "% % %\n" vx vy vz to: Fichier
	)

	format "ntvertex %\n" numtverts to: Fichier
	
	for i=1 to numtverts do
	(
		tcoord = getTVert tmesh i
		tx = tcoord.x
		ty = tcoord.y
		tz = tcoord.z
		format "% % %\n" tx ty tz to: Fichier
	)
	
	format "nfaces %\n" numfaces to: Fichier
	
	for i = 1 to numfaces do
	(
		face = getFace tmesh i
		
		format "% % %   " (int(face.x)-1) (int(face.y)-1) (int(face.z)-1) to: Fichier
		
		tvface = getTVFace tmesh i
		
		format "% % %\n" (int(tvface.x)-1) (int(tvface.y)-1) (int(tvface.z)-1) to: Fichier

		normals = meshop.getFaceRNormals tmesh i
		
		f_n = getFaceNormal tmesh i
		
		for j = 1 to 3 do
		(
			n=normals[j]
			try
			(
				format "% % %\n" (in coordsys world n.x) (in coordsys world n.y) (in coordsys world n.z) to: Fichier
			)
			catch
			(
				format "% % %\n" (in coordsys world f_n.x) (in coordsys world f_n.y) (in coordsys world f_n.z) to: Fichier
			)
		)
		format "% % %\n" (in coordsys world f_n.x) (in coordsys world f_n.y) (in coordsys world f_n.z) to: Fichier
				
	)

)

--Interface Graphique
	group "Export"
	(
		--Interface Graphique du groupe Settings	
		Button ExportButton "Export selected model" width:140 enabled:true
	)
			
	on ExportButton pressed do	
	(
		if selection.count >= 1 then
		(
			NomScene = "export"
			NomFichier = getSaveFileName caption:"TXT Exporter Save as..." filename: NomScene types:"txt (*.txt)|*.txt|All|*.*|" 

			if NomFichier != undefined do
			(
				Fichier = createFile NomFichier
				
				ExportMaterials SceneMaterials Fichier


				format "nobject %\n\n" selection.count to: Fichier
				
				for i=1 to selection.count do
				(
				    Object = selection[i]			
					Export Object SceneMaterials Fichier
					
					format "\n" to: Fichier
				)
	
				Close Fichier
		
				ShellLaunch "notepad.exe" NomFichier
				
--				max utility mode
--				select Model
			)
		)
		else
		(
			messagebox "Empty selection.\n" title:"TXT Export"
		)
	)
	
)