////////////////////////////////////////////////////////////////////////////////////
/// @file CMateriel.h
/// @author DGI
/// @date 2005-06-15
/// @version 1.0
///
/// @addtogroup chargeur3ds Chargeur 3DS
/// @{
////////////////////////////////////////////////////////////////////////////////////
#ifndef __CMATIERE_H__
#define __CMATIERE_H__


#include <string>

#include "Vecteur.h"
#include "CModelTexture.h"

using namespace std;


namespace chargeur3ds {


   ///////////////////////////////////////////////////////////////////////////
   /// @class CMateriel
   /// @brief 
   ///
   /// Contient un matriel dfini par les paramtres suivants:
   ///        Une couleur
   ///        Une texture
   ///        Un nom
   ///
   /// @author DGI-2990
   /// @date 2005-09-27
   ///////////////////////////////////////////////////////////////////////////
   class CMateriel
   {
   private:
      /// Materiel utilis lorsqu'aucun matriel n'est spcifi pour une surface.
      static CMateriel g_sansMateriel;
   public:
      /// Retourne le materiel par dfaut lorsqu'aucun n'est spcifi.
      static CMateriel* GetSansMateriel() { return &g_sansMateriel; }

       /// Construction 
       CMateriel(); 
       /// Destruction
       ~CMateriel();

       /// Surcharge de l'oprateurs < (crire dans un fichier binaire)
       friend CEcritureFichierBinaire& operator< ( CEcritureFichierBinaire& out, const CMateriel& m);
       /// Surcharge de l'oprateurs > (Lecture dans un fichier binaire)
       friend CLectureFichierBinaire& operator> ( CLectureFichierBinaire& in, CMateriel& m);

       /// Charge la texture en mmoire opengl
       void Load(const char* relatif = ".");

       /// Libre la texture
       void Remove();

       /// Active le matriel
       void Activate() const;

       /// Lit les paramtres du matriel  partir d'un fichier
       void ImportMaterial ( ifstream& in);

       /// Retourne true si le matriel est un miroir
       bool Reflective() const {return name=="miroir";}

       /// Texture
       CModelTexture map;

       /// Nom de la texture
       string name;

       /// Couleur du materiel
       Vecteur4f color;
   };


} // Fin de l'espace de nom chargeur3ds


#endif // __CMATIERE_H__


////////////////////////////////////////////////////////////////////////////////////
/// @}
////////////////////////////////////////////////////////////////////////////////////
