////////////////////////////////////////////////////////////////////////////////////
/// @file CModel.h
/// @author DGI
/// @date 2005-06-15
/// @version 1.0
///
/// @addtogroup chargeur3ds Chargeur 3DS
/// @{
////////////////////////////////////////////////////////////////////////////////////
#ifndef __CMODEL_H__
#define __CMODEL_H__


#include "Vecteur.h"

#include "CLectureFichierBinaire.h"
#include "CEcritureFichierBinaire.h"
#include "CObjet.h"
class CMateriel;
using namespace std;


namespace chargeur3ds {


   ///////////////////////////////////////////////////////////////////////////
   /// @class CModel
   /// @brief Classe reprsentant un modle 3DS.
   ///
   /// @author DGI-2990
   /// @date 2005-09-27
   ///////////////////////////////////////////////////////////////////////////


   class CModel
   {
   public:
       /// Construction 
       CModel();
       /// Destruction
       ~CModel();

       /// Surcharge de l'oprateur < (crire dans un fichier binaire)
       friend              CEcritureFichierBinaire& operator< ( CEcritureFichierBinaire& out, const CModel& mod);
       /// Surcharge de l'oprateur > (Lecture dans un fichier binaire)
       friend              CLectureFichierBinaire& operator> ( CLectureFichierBinaire& in, CModel& mod);

       /// Exprime les sommets par rapport au pivot (effectue une translation de vecteur t = -pivot)
       void                exprimerParRapportAuPivot();

       /// Importe la gometrie  partir d'un fichier
       void                importer ( const char* fname);

       /// Charge un modle  partir d'un fichier en regardant si le fichier binaire existe.
       void                Load ( string nom_modele, const char* fname, const char* relatif = ".");

       /// crit le modle sous forme de fichier binaire
       void                Write ( const char* fname) const;

       /// Met  jour le pointeur matriel de chacun des objets
       void                ecrirePointeurMateriel();

       /// Effectue une mise  l'chelle de tous les objets
       void                mettreAEchelle ( float f);

       /// Effectue une rotation de tous les objets
       void                tourner( float angle, const Vecteur3& axe );

       /// Dessine la gomtrie sous forme de points
       void                dessinerPoint() const;

       /// Dessine la gomtrie
       void                dessiner() const;

       /// Charge les matriaux
       void                chargerMateriaux();

       /// Efface les matriaux
       void                effacerMateriaux();

       /// Dessine un objet en particulier
       void                dessinerObjet ( int id) const;

       /// Renvoie une liste d'index correspondants aux objets rflchissants
       vector<int>         GetReflectiveId() const;

       /// Cherche un objet
       CObjet*             chercher (const string& nom_objet);

       /// Cherche un objet (version constante)
       const CObjet*       chercher (const string& nom_objet) const;

       /// Calcule la sphre englobante
       double              calculerSphereEnglobante() const;

       /// Calcule le cylindre englobant
       void                calculerCylindreEnglobant(double& rayon, double& bas, double& haut) const;

       /// Calcule le rayon du cylindre englobant
       void                calculerBoiteEnglobante(Vecteur3& coinMin, Vecteur3& coinMax) const;

       /// Vecteur d'objets
       vector<CObjet>      vecteurObjets_;

       /// Vecteur de matriaux
       vector<CMateriel>   vecteurMateriaux_;

       /// Nom du modle
       string              nom_;

       /// Rpertoire relatif du modle et des textures
       string              relatif_;
   };


} // Fin de l'espace de nom chargeur3ds


#endif // __CMODEL_H__


////////////////////////////////////////////////////////////////////////////////////
/// @}
////////////////////////////////////////////////////////////////////////////////////
