////////////////////////////////////////////////////////////////////////////////////
/// @file CModelTexture.cpp
/// @author DGI
/// @date 2005-06-15
/// @version 1.0
///
/// @addtogroup chargeur3ds Chargeur 3DS
/// @{
////////////////////////////////////////////////////////////////////////////////////

#ifdef WIN32
#include <windows.h>
#endif

#include <GL/gl.h>

#include "CModelTexture.h"
#include "AideGL.h"

namespace chargeur3ds {

   int CModelTexture::WrapType = 0;

   ////////////////////////////////////////////////////////////////////////
   ///
   /// @fn CModelTexture::CModelTexture ( const string& _tname)
   ///
   /// Constructeur par paramtres
   ///
   /// @param[in] _tname : Nom de la texture
   ///
   /// @return Aucune
   ///
   ////////////////////////////////////////////////////////////////////////
   CModelTexture::CModelTexture ( const string& _tname)
   :tname ( _tname), textureid ( 0)
   {
   }

   ////////////////////////////////////////////////////////////////////////
   ///
   /// @fn CModelTexture::SetWrapType ( int wt)
   ///
   /// Permet de mettre  jour le mode de wrap
   ///
   /// @param wt : Mode d'enveloppement 
   ///
   /// @return Aucune
   ///
   ////////////////////////////////////////////////////////////////////////
   void CModelTexture::SetWrapType ( int wt)
   {
       WrapType = wt;
   }

   ////////////////////////////////////////////////////////////////////////
   ///
   /// @fn operator< ( CEcritureFichierBinaire& out, const CModelTexture& smt)
   ///
   /// Surcharge de l'oprateur <
   ///
   /// @param[out] out : Flux d'criture
   /// @param[in] smt : Modle de texture  crire
   ///
   /// @return Flux d'criture
   ///
   ////////////////////////////////////////////////////////////////////////
   CEcritureFichierBinaire& operator< ( CEcritureFichierBinaire& out, const CModelTexture& smt)
   {
       out<smt.tname;
       return out;
   }

   ////////////////////////////////////////////////////////////////////////
   ///
   /// @fn operator> ( CLectureFichierBinaire& in, CModelTexture& smt)
   ///
   /// Surcharge de l'oprateur >
   ///
   /// @param[in] in : Flux de lecture
   /// @param[out] smt : Texture  lire
   ///
   /// @return Flux de lecture
   ///
   ////////////////////////////////////////////////////////////////////////
   CLectureFichierBinaire& operator> ( CLectureFichierBinaire& in, CModelTexture& smt)
   {
       in>smt.tname;
       return in;
   }

   ////////////////////////////////////////////////////////////////////////
   ///
   /// @fn CModelTexture::ImportFname ( ifstream& in)
   ///
   /// Importe les paramtres  partir d'un fichier
   ///
   /// @param[in] in : Flux de lecture
   ///
   /// @return Aucune
   ///
   ////////////////////////////////////////////////////////////////////////
   void CModelTexture::ImportFname ( ifstream& in)
   {
       string texturename;
       in>>texturename;
       tname  = string ("");
       tname += texturename;
   }

   ////////////////////////////////////////////////////////////////////////
   ///
   /// @fn CModelTexture::Load(const char* relatif)
   ///
   /// Fonction de chargement de la texture en mmoire opengl
   ///
   /// @return Aucune
   ///
   ////////////////////////////////////////////////////////////////////////
   void CModelTexture::Load(const char* relatif)
   {
       if  ( tname.size() > 3)  {
           string fullname = string(relatif) + string("/") + tname;
           aidegl::glLoadTexture ( fullname.c_str(), textureid );
           glBindTexture ( GL_TEXTURE_2D,textureid);

           if (WrapType) {
              glTexParameteri ( GL_TEXTURE_2D, GL_TEXTURE_WRAP_S, WrapType);
              glTexParameteri ( GL_TEXTURE_2D, GL_TEXTURE_WRAP_T, WrapType);
           }
       }
   }

   ////////////////////////////////////////////////////////////////////////
   ///
   /// @fn CModelTexture::Remove()
   ///
   /// Enlve la texture
   ///
   /// @return Aucune
   ///
   ////////////////////////////////////////////////////////////////////////
   void CModelTexture::Remove()
   {
       glDeleteTextures ( 1,&textureid);
   }

}  // Fin de l'espace de nom chargeur3ds


////////////////////////////////////////////////////////////////////////////////////
/// @}
////////////////////////////////////////////////////////////////////////////////////
