#ifdef _WIN32
#include <windows.h>
#endif
#include <GL/gl.h>
#include <GL/glu.h>
#include <GL/glut.h>
#include <math.h>
#include <iostream>

// Les modeles 3D
#include "CFace.h"
#include "CModel.h"

using namespace std;

// Variables pour l'orientation de la scene.
float          g_Yaw    = 0.0f;
float          g_Pitch  = -100.0f;
float          g_Zoom   = 600.0f;

chargeur3ds::CModel  g_leModele;
chargeur3ds::CObjet *g_laBoite;
chargeur3ds::CObjet *g_laRoue1;
chargeur3ds::CObjet *g_laRoue2;

const float    g_distanceRotation = 200.0f;
float          g_rotationModele = 0.0f;
float          g_rotationRoue = 0.0f;

unsigned int   g_listid_;

// Variables pour gerer le deplacement de la souris.
int            g_MouseX, g_MouseY, g_Button;

// Conserve la profondeur d'un point selectionne.
float          g_PointZ;

enum           Mode { MODE_VIEW, MODE_POINT }    g_Mode = MODE_VIEW;

void Display()
{
    // Efface le tampon image.
    glClear( GL_COLOR_BUFFER_BIT | GL_DEPTH_BUFFER_BIT );

    // Place l'observateur.
    glLoadIdentity();
    float x    = g_Zoom * cos( g_Pitch ) * sin( g_Yaw );
    float y    = g_Zoom * cos( g_Pitch ) * cos( g_Yaw );
    float z    = g_Zoom * sin( g_Pitch );
    gluLookAt( x, y, z, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0 );

    // Dessine les axes.
    glDisable( GL_LIGHTING );
    glBegin( GL_LINES );
    glColor3f( 1.0f, 0.0f, 0.0f );
    glVertex3f( 0.0f, 0.0f, 0.0f );
    glVertex3f( 100.0f, 0.0f, 0.0f );

    glColor3f( 0.0f, 1.0f, 0.0f );
    glVertex3f( 0.0f, 0.0f, 0.0f );
    glVertex3f( 0.0f, 100.0f, 0.0f );

    glColor3f( 0.0f, 0.0f, 1.0f );
    glVertex3f( 0.0f, 0.0f, 0.0f );
    glVertex3f( 0.0f, 0.0f, 100.0f );
    glEnd();
    glEnable( GL_LIGHTING );

    glEnable(GL_TEXTURE_2D);

    // Affichage du vhicule
    glPushMatrix();
    {
        // Rotation et dplacement du vhicule
        // (Rsultat : le vhicule tourne  une distance "g_distanceRotation" autour de l'axe vertical (z)
        glRotatef(g_rotationModele,0,0,1);
        glTranslatef(g_distanceRotation,0,0);

        // Affichage de la bote en la replaant  son pivot
        glPushMatrix();
        {
            glTranslatef(g_laBoite->positionPivot[0], g_laBoite->positionPivot[1], g_laBoite->positionPivot[2]);
            g_laBoite->dessiner();
        }
        glPopMatrix();

        // Affichage de la premire roue en la replaant  son pivot puis
        // en effectuant la rotation pour simuler que la roue tourne.
        glPushMatrix();
        {
            glTranslatef(g_laRoue1->positionPivot[0], g_laRoue1->positionPivot[1], g_laRoue1->positionPivot[2]);
            glRotatef(g_rotationRoue,-1,0,0);
            g_laRoue1->dessiner();
        }
        glPopMatrix();

        // Affichage de la seconde roue en la replaant  son pivot puis
        // en effectuant la rotation pour simuler que la roue tourne.
        glPushMatrix();
        {
            glTranslatef(g_laRoue2->positionPivot[0], g_laRoue2->positionPivot[1], g_laRoue2->positionPivot[2]);
            glRotatef(g_rotationRoue,-1,0,0);
            g_laRoue2->dessiner();
        }
        glPopMatrix();
    }
    glPopMatrix();

    glDisable(GL_TEXTURE_2D);

    // Affiche le tampon image.
    glutSwapBuffers();
}


void Reshape( int width, int height )
{
    // Ajuste la matrice de projection.
    glMatrixMode( GL_PROJECTION );
    glLoadIdentity();
    gluPerspective( 60.0, (double)width / height, 10.0, 3000.0 );
    glMatrixMode( GL_MODELVIEW );
}

void OnMouseButton( int Button, int State, int x, int y )
{
    g_MouseX = x;
    g_MouseY = y;
    g_Button = Button;
}


void OnMouseMove( int x, int y )
{
    float DiffX = g_MouseX - x;
    float DiffY = g_MouseY - y;

    switch( g_Mode )
    {
    case MODE_VIEW:
        {
            // Bouton de gauche en mode vue...
            if( g_Button == GLUT_LEFT_BUTTON )
            {
                // ...on change l'orientation.
                g_Pitch    += DiffY * 0.01f;
                g_Yaw    += DiffX * 0.01f;

                // On affiche la scene.
                glutPostRedisplay();
            }
            // Bouton de droite en mode vue...
            else if( g_Button == GLUT_RIGHT_BUTTON )
            {
                // ...on change la distance a l'origine.
                float    k;
                if( DiffX + DiffY > 0 )
                    k = sqrt( DiffX * DiffX + DiffY * DiffY );
                else
                    k = -sqrt( DiffX * DiffX + DiffY * DiffY );

                if( g_Zoom * ( 1 + k * 0.01 ) > 0.0 )
                    g_Zoom    *= 1 + k * 0.01;

                // On affiche la scene.
                glutPostRedisplay();
            }
        }
        break;        
    }

    g_MouseX = x;
    g_MouseY = y;
}


void OnKeyPressed( unsigned char Key, int x, int y )
{
    switch( Key )
    {
    case 27:
        // ESC.
        exit( 0 );
    }
}


void Init()
{
    // Initialisation des etats d'OpenGL.
    glClearColor( 0.0f, 0.0f, 0.0f, 0.0f );
    glClearDepth( 1.0f );
    glEnable( GL_DEPTH_TEST );
    glDepthFunc( GL_LEQUAL );
    glEnable( GL_LIGHTING );
    glEnable( GL_LIGHT0 );
    glEnable( GL_COLOR_MATERIAL );

    // Chargement du modle et des matriaux.
    g_leModele.Load ( "vehicule", "vehicule" );
    g_leModele.chargerMateriaux();

    // Comme les objets seront affichs un  un, demander  ce que le pointeur
    // des matriaux soit affect par objet.
    g_leModele.ecrirePointeurMateriel();

    // Les dplacements et rotations seront faites pour chaque objet sparment.
    // Il faut donc retirer le pivot de la position des points des objets.
    g_leModele.exprimerParRapportAuPivot();

    // Obtention de pointeurs vers chaque objet composant le modle.
    g_laBoite = g_leModele.chercher("boite");
    g_laRoue1 = g_leModele.chercher("roue1");
    g_laRoue2 = g_leModele.chercher("roue2");
}

void Timer(int)
{
    // Mise  jour des rotations.

    g_rotationModele += 2.0f;
    while (g_rotationModele > 360.0f) g_rotationModele -= 360.0f;

    g_rotationRoue += 5.0f;
    while (g_rotationRoue > 360.0f) g_rotationRoue -= 360.0f;

    glutPostRedisplay();
    glutTimerFunc( 30, Timer, 0 );
}

int main( int argc, char** argv )
{
    glutInit( &argc, argv );
    glutInitWindowSize( 600, 600 );
    glutInitDisplayMode( GLUT_RGBA | GLUT_DOUBLE | GLUT_DEPTH );
    glutCreateWindow( "Chargeur3DS" );
    Init();
    glutDisplayFunc( Display );
    glutKeyboardFunc( OnKeyPressed );
    glutMouseFunc( OnMouseButton );
    glutMotionFunc( OnMouseMove );
    glutReshapeFunc( Reshape );
    glutTimerFunc( 30, Timer, 0 );
    glutMainLoop();

    return 0;
}
